Attribute VB_Name = "CreateBatchFiles"
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Create_Batch_files()
'
' This procedure creates the batch files generated by HardenNT.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Create_Batch_files(startbatchfile As String, _
                       enablepatchessecurity As Boolean, _
                       enableNTsecurity As Boolean, _
                       enableNTFSsecurity As Boolean, _
                       enableregistrysecurity As Boolean, _
                       patchesbatchfile As String, _
                       ntbatchfile As String, _
                       ntfsbatchfile As String, _
                       registrybatchfile As String, _
                       logfile As String, _
                       adminaccount As String, _
                       adminpassword As String, _
                       Platform As String, _
                       OS As String, _
                       Profile As String, _
                       patcheslistview As ListView, _
                       userrightslistview As ListView, _
                       auditinglistview As ListView, _
                       ntfslistview As ListView, _
                       registrylistview As ListView, _
                       patcheslocation As String, _
                       ntfsrescuefolderlocation As String, _
                       systemfilesdrive As String, _
                       connection As ADODB.connection, _
                       administratoraccount As String, administratorpassword As String)
    On Error GoTo Create_Batch_Files_Error:

    Dim Eval_Batch, Eval_Log, Eval_Admin As String
    Dim ini_file_name As String
    Dim nextfiletocall As String
    Dim fso1, ts1
    Dim strippedfilename As String
    Dim inifilename As String
    Dim nextbatchfile As String
    Dim cleanupinifile As String
    Dim cleanupbatchfile As String

    ' Create NT Patches Batch File
    ' Open File
    Set fso1 = CreateObject("Scripting.FileSystemObject")
    Set ts1 = fso1.opentextfile(startbatchfile, 2, True)
        
    Eval_Batch = (patchesbatchfile <> "")
    Eval_Log = (logfile <> "")
    Eval_Admin = (adminaccount <> "")

    If Eval_Batch And Eval_Log And Eval_Admin Then
        ' Write date and time to logfile
        ts1.writeline ("@echo off")
        ts1.writeline ("echo '*** Hardening " & Platform & " " & OS & _
                       " " & " ***' >> " & logfile)
        ts1.writeline ("date /t >> " & logfile)
        ts1.writeline ("time /t >> " & logfile)

        ' Profile info
        ts1.writeline
        If Profile <> "" Then
            ts1.writeline ("echo 'Profile used to secure target host: " & Profile & "' >> " & logfile)
        Else
            ts1.writeline ("echo 'No Profile used to create bath files' >> " & logfile)
        End If
        ts1.writeline

        ' Will the system need a reboot ?
        ' -> yes if patches and/or registry tweaks are applied
        ' -> if the system is rebooted, make sure to autologon after the reboot(s)
        If enablepatchessecurity Or enableregistrysecurity Then
            ' Automate logon process after reboot : Create .INI file
            ' Set filenames
            strippedfilename = Left(patchesbatchfile, InStr(patchesbatchfile, ".bat") - 1)
            cleanupbatchfile = strippedfilename + "-cleanup.bat"
            cleanupinifile = strippedfilename + "-cleanup.ini"
            inifilename = strippedfilename + "-autologon.ini"
        
            nextbatchfile = cleanupbatchfile
            If enableregistrysecurity = True Then nextbatchfile = registrybatchfile
            If enableNTFSsecurity = True Then nextbatchfile = ntfsbatchfile
            If enableNTsecurity = True Then nextbatchfile = ntbatchfile

            Call Create_Autologon_File(inifilename, nextbatchfile, _
                                       administratoraccount, administratorpassword)

            ' Call REGINI to set autologon registry values
            ts1.writeline ("echo 'Setting Autologon options for after reboot' >> " & logfile)
            ts1.writeline ("regini " & inifilename & " >> " & logfile)
            ts1.writeline
        End If

        '''''''''''
        ' PATCHES
        '''''''''''

        ' Create Patches Batch File
        If enablepatchessecurity = True Then
            ' Call Patch script
            ts1.writeline ("echo 'Running the next batch file: " & patchesbatchfile & "' >> " & logfile)
            ts1.writeline ("CALL " & patchesbatchfile)
            
            ' Reboot the system
            ts1.writeline
            ts1.writeline ("echo 'Rebooting the system' >> " & logfile)
            ts1.writeline ("shutdown /L /R /Y /C")
            ts1.Close
        
            Call Create_Patch_Batch_File(patcheslistview, _
                                         patchesbatchfile, _
                                         logfile, _
                                         patcheslocation, _
                                         "Patch_Executable", _
                                         "MS_Fix_ID", _
                                         "MS_Fix_Description")
        End If
        
        '''''''''''''''
        ' NT SECURITY
        '''''''''''''''

        ' Next file to call
        nextbatchfile = cleanupbatchfile
        If enableregistrysecurity Then nextbatchfile = registrybatchfile
        If enableNTFSsecurity Then nextbatchfile = ntfsbatchfile

        ' Create NT Security Batch File
        If enableNTsecurity = True Then
            Call Create_NT_Security_Batch_File(ntbatchfile, _
                                               logfile, _
                                               userrightslistview, _
                                               auditinglistview, _
                                               nextbatchfile, _
                                               "User_Rights_Operation_Name", _
                                               "NT_User_Group_Name", _
                                               "User_Rights_Privilege_Name")
        End If

        '''''''''''''''''
        ' NTFS SECURITY
        '''''''''''''''''

        ' Next file to call
        If enableregistrysecurity Then
            nextbatchfile = registrybatchfile
        Else
            If cleanupbatchfile <> "" Then
                nextbatchfile = cleanupbatchfile
            Else
                nextbatchfile = ""
            End If
        End If

        ' Create NTFS Permissions Batch File
        If enableNTFSsecurity = True Then
            Call Create_NTFS_Permissions_Batch_File(ntfsbatchfile, _
                                                    logfile, _
                                                    ntfslistview, _
                                                    nextbatchfile, _
                                                    ntfsrescuefolderlocation, _
                                                    systemfilesdrive, _
                                                    "NTFS_Operation_ID", _
                                                    "NTFS_Folder", _
                                                    "NTFS_File", _
                                                    "NTFS_Recursively", _
                                                    "NTFS_Set_Perms_Type_ID", _
                                                    "NTFS_Permission_Administrators", _
                                                    "NTFS_PERMISSION_SYSTEM", _
                                                    "NTFS_Permission_Users", _
                                                    "NTFS_Permission_Guests", _
                                                    "NTFS_PERMISSION_EVERYONE", _
                                                    "NTFS_PERMISSION_AUTHENTICATED_USERS", _
                                                    "NTFS_PERMISSION_CREATOR_OWNER", _
                                                    "NTFS_PERMISSION_NETWORK", _
                                                    "NTFS_PERMISSION_INTERACTIVE", _
                                                    "NTFS_Permission_Server_Operators", _
                                                    "NTFS_Permission_Account_Operators", _
                                                    "NTFS_Permission_Backup_Operators", _
                                                    "NTFS_Permission_Print_Operators", _
                                                    "NTFS_Permission_Power_Users")
        End If

    '''''''''''''''''''''
    ' REGISTRY SECURITY
    '''''''''''''''''''''

        ' Next file to call
        If cleanupbatchfile <> "" Then
            nextbatchfile = cleanupbatchfile
        Else
            nextbatchfile = ""
        End If

        ' Create Registry Batch File
        If enableregistrysecurity = True Then
            ini_file_name = Left(registrybatchfile, InStr(registrybatchfile, ".bat") - 1)
            ini_file_name = ini_file_name + ".ini"

            Call Create_Registry_Batch_File(registrybatchfile, logfile, ini_file_name, connection, registrylistview, nextbatchfile)
        End If

        '''''''''''
        ' CLEANUP
        '''''''''''

        ' Create Cleanup Batch File
        ' Only if the system has been rebooted with an automatic userlogon
        If enablepatchessecurity Or enableregistrysecurity Then
            Call Create_Cleanup_Batch_File(cleanupbatchfile, cleanupinifile, logfile)
        End If
    
        MsgBox "Batch Files created. Run " & patchesbatchfile & " on Target Machine"
    Else
        MsgBox "Please fill in a name for the Batch File, the Log File and " & _
               "the Administrator Account"
    End If

    Exit Sub

Create_Batch_Files_Error:
    Call Error_Message("Create_Batch_Files()")
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Create_Patch_Batch_File()
'
' Creates a batch file that will run a number of NT patches on a target
' host.
' 'lsvw' is the listview that contains a list of patch file names.
' 'BatchFileName' is the name of the batch file that will be created.
' 'log_file_name' is the name of the logfile that will contain all output
' generated by the 'BatchFileName' batch file.
' 'patches_exe_column_name', 'patches_MS_ID_column_name' and
' 'patches_descr_column_name' are used to determine which columns contain
' the names of the executables, the MS_ID and a description.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Create_Patch_Batch_File(lsvw As ListView, _
                            BatchFileName As String, _
                            log_file_name As String, _
                            location_name As String, _
                            patches_exe_column_name As String, _
                            patches_MS_ID_column_name As String, _
                            patches_descr_column_name As String)
    Dim fso1, ts1
    Dim i As Integer
    Dim i_exe, i_MS_ID, i_descr As Integer

    ' Open File
    Set fso1 = CreateObject("Scripting.FileSystemObject")
    Set ts1 = fso1.opentextfile(BatchFileName, 2, True)
        
    ts1.writeline ("@echo off")
    ts1.writeline ("echo '*** Executing Security Patches on Host, running " & BatchFileName & " ***' >> " & log_file_name)
    ts1.writeline

    ' Write Patches to be installed to file

    ' Which column contains name of executable, MS_ID and description ?
    i_exe = GetListviewColumnIndex(lsvw, patches_exe_column_name) - 1
    i_MS_ID = GetListviewColumnIndex(lsvw, patches_MS_ID_column_name) - 1
    i_descr = GetListviewColumnIndex(lsvw, patches_descr_column_name) - 1

    For i = 1 To lsvw.ListItems.Count
        If lsvw.ListItems(i).Checked = True Then
            If i_MS_ID = 0 Then
                ts1.writeline ("echo '" & lsvw.ListItems(i).text & _
                               " - " & lsvw.ListItems(i).SubItems(i_descr) & ", Running " & location_name & _
                               lsvw.ListItems(i).SubItems(i_exe) & "' >> " & _
                               log_file_name)
            Else
                ts1.writeline ("echo '" & lsvw.ListItems(i).SubItems(i_MS_ID) & _
                               " - " & lsvw.ListItems(i).SubItems(i_descr) & ", Running " & location_name & _
                               lsvw.ListItems(i).SubItems(i_exe) & "' >> " & _
                               log_file_name)
            End If
            If (InStr(lsvw.ListItems(i).SubItems(i_exe), "***")) Then
                ts1.writeline ("echo 'This patch is currently unavailable from the MS site' >> " & _
                               log_file_name)
                ts1.writeline ("wait 5")
            Else
                ts1.writeline (location_name & lsvw.ListItems(i).SubItems(i_exe) & _
                               " -m -q -z >> " & log_file_name)
            End If
            ts1.writeline
        End If
    Next
            
    ' Close File
    ts1.Close

End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Create_NT_Security_Batch_file()
'
' Creates a batch file ('file_name') that performs the following two tasks :
' - Set NT Privileges as described by the UR_Listview listview
' - Set Auditing for certain events as described by the A_Listview listview
' Parameters passed:
' - 'file_name' contains the name of the batch file to create.
' - 'log_file_name' contains the name of the logfile to be created when the
' batch file is run.
' - 'UR_Listview' is the listview containing the user right entries.
' - 'A_Listview' is the listview containing the auditing entries.
' - 'nextfiletocall' contains the name of the batch file to call after execution
' of 'file_name'.
' - 'UR_column_name', 'NT_group_column_name', and 'Privilege_column_name'
' contain the names of the columnheaders of the columns for the user right,
' NT group name and privilege name respectively.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Create_NT_Security_Batch_File(file_name As String, _
                                  log_file_name, _
                                  UR_Listview As ListView, _
                                  A_Listview As ListView, _
                                  nextfiletocall As String, _
                                  UR_column_name As String, _
                                  NT_group_column_name As String, _
                                  Privilege_column_name As String)
    Dim fso, fs
    Dim i, j As Integer
    Dim UR_Operation_Column As Integer
    Dim UR_Group_Name_Column As Integer
    Dim UR_Privilege_Column As Integer

    Set fso = CreateObject("Scripting.FileSystemObject")
    Set fs = fso.opentextfile(file_name, 2, True)
    
    '''''''''''''''''''''''''''
    ' First set NT User Rights
    '''''''''''''''''''''''''''
    fs.writeline ("@echo off")
    fs.writeline ("echo '*** Setting NT User Rights, Running " & file_name & " ***' >> " & log_file_name)
    fs.writeline

    UR_Operation_Column = GetListviewColumnIndex(UR_Listview, UR_column_name) - 1
    UR_Group_Name_Column = GetListviewColumnIndex(UR_Listview, NT_group_column_name) - 1
    UR_Privilege_Column = GetListviewColumnIndex(UR_Listview, Privilege_column_name) - 1
    
    For i = 1 To UR_Listview.ListItems.Count
        ' Checked
        If UR_Listview.ListItems(i).Checked Then
            ' Operation ?
            Select Case UR_Listview.ListItems(i).SubItems(UR_Operation_Column)
                Case "ADD USER RIGHT"
                    operation_param = "+r"
                Case "REVOKE USER RIGHT"
                    operation_param = "-r"
            End Select
            
            ' Groups
            If UR_Listview.ListItems(i).SubItems(UR_Group_Name_Column) = UCase(UR_Listview.ListItems(i).SubItems(UR_Group_Name_Column)) Then
                ' Don't change anything
                fs.writeline ("ntrights -u " & Chr(34) & _
                              UR_Listview.ListItems(i).SubItems(UR_Group_Name_Column) & Chr(34) & _
                              " " & operation_param & " " & UR_Listview.ListItems(i).SubItems(UR_Privilege_Column) & " >> " & log_file_name)
            Else
                ' Substitute group for one or more real groups
                For j = 0 To ManageUserGroups.ReturnNumberOfSubGroups(ManageUserGroups.ReturnNTGroupsArrayIndex(UR_Listview.ListItems(i).SubItems(UR_Group_Name_Column))) - 1
                    fs.writeline ("ntrights -u " & Chr(34) & _
                                  ManageUserGroups.ReturnSubGroupName(ManageUserGroups.ReturnNTGroupsArrayIndex(UR_Listview.ListItems(i).SubItems(UR_Group_Name_Column)), j) & Chr(34) & _
                                  " " & operation_param & " " & UR_Listview.ListItems(i).SubItems(UR_Privilege_Column) & " >> " & log_file_name)
                Next
            End If
        End If
    Next

    ''''''''''''''''''''''''''''
    ' Next set Auditing options
    ''''''''''''''''''''''''''''
    fs.writeline
    fs.writeline ("echo '*** Setting Auditing, Running " & file_name & " ***' >> " & log_file_name)
    fs.writeline

    ' Enable auditing
    fs.writeline ("echo 'Enabling Auditing' >> " & log_file_name)
    fs.writeline ("auditpol /enable >> " & log_file_name)

    For i = 1 To A_Listview.ListItems.Count
        If A_Listview.ListItems(i).Checked Then
            If (i Mod 2) And (i <= A_Listview.ListItems.Count) Then
                If A_Listview.ListItems(i + 1).Checked Then
                    fs.writeline ("auditpol /enable /" & A_Listview.ListItems(i).SubItems(1) & ":All >> " & log_file_name)
                    i = i + 1
                Else
                    fs.writeline ("auditpol /enable /" & A_Listview.ListItems(i).SubItems(1) & ":" & A_Listview.ListItems(i).SubItems(2) & " >> " & log_file_name)
                End If
            Else
                fs.writeline ("auditpol /enable /" & A_Listview.ListItems(i).SubItems(1) & ":" & A_Listview.ListItems(i).SubItems(2) & " >> " & log_file_name)
            End If
        End If
    Next

    ' Call the next batch file
    fs.writeline
    fs.writeline ("echo '*** Calling the next batch file: " & nextfiletocall & " ***' >> " & log_file_name)
    fs.writeline (nextfiletocall)

    fs.Close

End Sub


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Create_NTFS_Permissions_Batch_File()
'
' Creates a batch file, 'file_name', that sets ACLs on files and folders
' via the XCALCS.EXE resource kit utility.
' This function calls Write_To_NTFS_Batch_File() to write the individual
' NTFS operations to the file 'file_name'.
' Parameters:
' - 'file_name' : contains the name of the batch file to create.
' - 'log_file_name' : contains the name of the logfile to log to at runtime.
' - 'N_Listview' : listview containing NTFS entries.
' - 'nextfiletocall' : batch file to call after NTFS batch file.
' - 'rescue_folder' : folder used to copy files to from their default location.
' - 'sysfiles_Drive' : drive letter of system partition.
' - 'NTFS_operation_ID_column_name' : type of operation (set NTFS perms, delete, ...).
' - 'NTFS_folder_column_name' : name column containing folder names.
' - 'NTFS_file_column_name' : name of column containing file names.
' - 'NTFS_recursively_column_name' : set NTFS perms recursively or not ?
' - 'NTFS_set_perms_type_ID_column_name' : REPLACE EXISTING or ADD TO EXISTING
' - 'NTFS_permissions_administrators_column_name',
'   'NTFS_permissions_system_column_name'
'   'NTFS_permissions_users_column_name,
'   'NTFS_permissions_guests_column_name',
'   'NTFS_permissions_everyone_column_name',
'   'NTFS_permissions_authenticated_users_column_name',
'   'NTFS_permissions_creator_owner_column_name',
'   'NTFS_permissions_network_column_name',
'   'NTFS_permissions_interactive_column_name',
'   'NTFS_permissions_server_operators_column_name',
'   'NTFS_permissions_account_operators_column_name',
'   'NTFS_permissions_backup_operators_column_name',
'   'NTFS_permissions_print_operators_colunm_name',
'   'NTFS_permissions_power_users_column_name' : names of columns containing
'   rights of administrators, system, etc...
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Create_NTFS_Permissions_Batch_File(file_name As String, _
                                       log_file_name As String, _
                                       N_listview As ListView, _
                                       nextfiletocall As String, _
                                       rescue_folder As String, _
                                       sysfiles_Drive As String, _
                                       NTFS_operation_ID_column_name As String, _
                                       NTFS_folder_column_name As String, _
                                       NTFS_file_column_name As String, _
                                       NTFS_recursively_column_name As String, _
                                       NTFS_set_perms_type_ID_column_name As String, _
                                       NTFS_permissions_administrators_column_name As String, _
                                       NTFS_permissions_system_column_name As String, _
                                       NTFS_permissions_users_column_name As String, _
                                       NTFS_permissions_guests_column_name As String, _
                                       NTFS_permissions_everyone_column_name As String, _
                                       NTFS_permissions_authenticated_users_column_name As String, _
                                       NTFS_permissions_creator_owner_column_name As String, _
                                       NTFS_permissions_network_column_name As String, _
                                       NTFS_permissions_interactive_column_name As String, _
                                       NTFS_permissions_server_operators_column_name As String, _
                                       NTFS_permissions_account_operators_column_name As String, _
                                       NTFS_permissions_backup_operators_column_name As String, _
                                       NTFS_permissions_print_operators_colunm_name As String, _
                                       NTFS_permissions_power_users_column_name As String)
    Dim fso, fs, i, j
    Dim tli As ListItem

    Set fso = CreateObject("scripting.filesystemobject")
    Set fs = fso.opentextfile(file_name, 2, True)

    fs.writeline ("@echo off")
    fs.writeline ("echo '*** Setting NTFS Permissions, running " & file_name & " ***' >> " & log_file_name)
    fs.writeline

    For i = 1 To N_listview.ListItems.Count
        Set tli = N_listview.ListItems(i)

        If tli.Checked Then
            Call Write_To_NTFS_Batch_File(fs, _
                                          log_file_name, _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_operation_ID_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_folder_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_file_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_recursively_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_set_perms_type_ID_column_name) - 1), _
                                          rescue_folder, _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_administrators_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_system_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_users_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_guests_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_everyone_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_authenticated_users_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_creator_owner_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_network_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_interactive_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_server_operators_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_account_operators_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_backup_operators_column_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_print_operators_colunm_name) - 1), _
                                          tli.SubItems(GetListviewColumnIndex(N_listview, NTFS_permissions_power_users_column_name) - 1), _
                                          sysfiles_Drive _
                                         )
        

        End If
    Next
    
    ' Call the next batch file
    fs.writeline ("echo '*** Calling the next batch file: " & nextfiletocall & " ***' >> " & log_file_name)
    fs.writeline (nextfiletocall)
    fs.Close
End Sub


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Create_Registry_Batch_File()
'
' Creates a batch file ('file_name') that runs REGINI.EXE on a .ini file
' ('inifilename'). inifilename is also created from the database registry
' entries in R_Listview
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Create_Registry_Batch_File(file_name As String, _
                               log_file_name As String, _
                               inifilename As String, _
                               db_connection As ADODB.connection, _
                               R_Listview As ListView, _
                               nextfiletocall As String)

    Dim perm(0 To 8) As Integer
    Dim dataconnection As New ADODB.Recordset
    Dim fso, fs, i, j

    Set fso = CreateObject("Scripting.FileSystemObject")
    Set ts = fso.opentextfile(inifilename, 2, True)

    For i = 1 To R_Listview.ListItems.Count
        If R_Listview.ListItems(i).Checked Then

            For j = 0 To 7
                dataconnection.Open "SELECT * FROM Registry_Permission", db_connection, adOpenDynamic
                dataconnection.MoveFirst
                
                Do Until (dataconnection.Fields.Item(1) = R_Listview.ListItems(i).SubItems(j + 6)) Or dataconnection.EOF
                    dataconnection.MoveNext
                Loop
            
                If dataconnection.Fields.Item(1) = R_Listview.ListItems(i).SubItems(j + 6) Then
                    perm(j) = dataconnection.Fields.Item(0)
                Else
                    perm(j) = 0
                End If
            
                dataconnection.Close
            Next
            Call Write_To_Regini_file(ts, _
                                      R_Listview.ListItems(i).SubItems(1), _
                                      R_Listview.ListItems(i).SubItems(2), _
                                      R_Listview.ListItems(i).SubItems(3), _
                                      R_Listview.ListItems(i).SubItems(4), _
                                      R_Listview.ListItems(i).SubItems(5), _
                                      perm(0), _
                                      perm(1), _
                                      perm(2), _
                                      perm(3), _
                                      perm(4), _
                                      perm(5), _
                                      perm(6), _
                                      perm(7))
        End If
    Next
    
    ts.Close

    Set ts = fso.opentextfile(file_name, 2, True)
    ts.writeline ("@echo off")
    ts.writeline ("echo '*** Tightening Registry, Running " & file_name & " ***' >> " & log_file_name)
    ts.writeline ("regini " & inifilename & " >> " & log_file_name)
    
    ' Call next batch file
    ts.writeline ("echo '*** Call the next batch file: " & nextfiletocall & " ***' >> " & log_file_name)
    ts.writeline (nextfiletocall)
    
    ts.Close

End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Create_Cleanup_Batch_File()
'
' This procedure creates a batch file that cleans up after the other batch
' files have run + the accompanying .ini file.
' Clean-up consists of deleting the registry entries for the automated
' logon.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Create_Cleanup_Batch_File(cleanupbatchfile As String, _
                              cleanupinifile As String, _
                              logfilename As String)
    Dim fso1, ts1
    Dim fso2, ts2

    Set fso1 = CreateObject("Scripting.FileSystemObject")
    Set ts1 = fso1.opentextfile(cleanupbatchfile, 2, True)
    
    Set fso2 = CreateObject("Scripting.FileSystemObject")
    Set ts2 = fso2.opentextfile(cleanupinifile, 2, True)

    Call Write_To_Regini_file(ts2, "DELETE KEY", "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon\", "DefaultUserName", "REG_SZ", "", 0, 0, 0, 0, 0, 0, 0, 0)
    Call Write_To_Regini_file(ts2, "DELETE KEY", "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon\", "DefaultPassword", "REG_SZ", "", 0, 0, 0, 0, 0, 0, 0, 0)
    Call Write_To_Regini_file(ts2, "ADD KEY", "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon\", "AutoAdminLogon", "REG_SZ", "0", 0, 0, 0, 0, 0, 0, 0, 0)
    Call Write_To_Regini_file(ts2, "DELETE KEY", "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\RunOnce\", "HardenNT", "REG_SZ", "", 0, 0, 0, 0, 0, 0, 0, 0)

    ts1.writeline ("@echo off")
    ts1.writeline ("echo '*** Cleaning up, Running " & cleanupbatchfile & " ***' >> " & logfilename)
    ts1.writeline ("regini " & cleanupinifile & " >> " & logfilename)

    ts1.Close
    ts2.Close
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Write_To_NTFS_Batch_File()
'
' This procedure write one NTFS entry to a batch file. This entry will
' delete, move, set permissions on a file/folder.
' Parameters:
' - ts: file handle
' - log_file_name: name of the logfile used by the batch file instructions
' - foldername: name of folder
' - filename: name of file
' - recursively: determines whether folder changes are to be set recursively
' - destination_folder: destination folder if operation is a MOVE
' - perms_...: permissions for various NT group classes (note that this
'   subroutine calls ReturnNumberOfSubGroups() and ReturnSubGroupName() from
'   the ManageUserGroups form)
' - drive_letter: drive letter of the system files like boot.ini, etc.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Write_To_NTFS_Batch_File(ts, _
                             log_file_name As String, _
                             operation As String, _
                             foldername As String, _
                             filename As String, _
                             recursively, _
                             replaceperms As String, _
                             destination_folder As String, _
                             perms_administrators As String, _
                             perms_SYSTEM As String, _
                             perms_users As String, _
                             perms_guests As String, _
                             perms_EVERYONE As String, _
                             perms_AUTHENTICATED_USERS As String, _
                             perms_CREATOR_OWNER As String, _
                             perms_NETWORK As String, _
                             perms_INTERACTIVE As String, _
                             perms_server_operators As String, _
                             perms_account_operators As String, _
                             perms_backup_operators As String, _
                             perms_print_operators As String, _
                             perms_power_users As String, _
                             drive_letter As String)
    Dim options As String
    Dim filepath As String
    Dim perms As String
    Dim t As Integer

    ' Parse path: Does it contain %boot%
    filepath = Replace(foldername, "%boot%", drive_letter)
    
    options = ""
    perms = " /G "
    If recursively = True Then
        options = options & " /T"
    End If

    Select Case replaceperms
        Case "REPLACE EXISTING"
            ' do nothing; default action is to replace permissions
        Case "ADD TO EXISTING"
            options = options & " /E"
    End Select

    If perms_administrators <> "***" Then
        For t = 0 To ManageUserGroups.ReturnNumberOfSubGroups(ManageUserGroups.ReturnNTGroupsArrayIndex("Administrators")) - 1
            perms = perms + " " + Chr(34) + ManageUserGroups.ReturnSubGroupName(ManageUserGroups.ReturnNTGroupsArrayIndex("Administrators"), t) + Chr(34) + ":" + GetNTFSPermissions(perms_administrators)
        Next t
    End If
    If perms_users <> "***" Then
        For t = 0 To ManageUserGroups.ReturnNumberOfSubGroups(ManageUserGroups.ReturnNTGroupsArrayIndex("Users")) - 1
            perms = perms + " " + Chr(34) + ManageUserGroups.ReturnSubGroupName(ManageUserGroups.ReturnNTGroupsArrayIndex("Users"), t) + Chr(34) + ":" + GetNTFSPermissions(perms_users)
        Next t
    End If
    If perms_guests <> "***" Then
        For t = 0 To ManageUserGroups.ReturnNumberOfSubGroups(ManageUserGroups.ReturnNTGroupsArrayIndex("Guests")) - 1
            perms = perms + " " + Chr(34) + ManageUserGroups.ReturnSubGroupName(ManageUserGroups.ReturnNTGroupsArrayIndex("Guests"), t) + Chr(34) + ":" + GetNTFSPermissions(perms_guests)
        Next t
    End If
    If perms_server_operators <> "***" Then
        For t = 0 To ManageUserGroups.ReturnNumberOfSubGroups(ManageUserGroups.ReturnNTGroupsArrayIndex("Server Operators")) - 1
            perms = perms + " " + Chr(34) + ManageUserGroups.ReturnSubGroupName(ManageUserGroups.ReturnNTGroupsArrayIndex("Server Operators"), t) + Chr(34) + ":" + GetNTFSPermissions(perms_server_operators)
        Next t
    End If
    If perms_account_operators <> "***" Then
        For t = 0 To ManageUserGroups.ReturnNumberOfSubGroups(ManageUserGroups.ReturnNTGroupsArrayIndex("Account Operators")) - 1
            perms = perms + " " + Chr(34) + ManageUserGroups.ReturnSubGroupName(ManageUserGroups.ReturnNTGroupsArrayIndex("Account Operators"), t) + Chr(34) + ":" + GetNTFSPermissions(perms_account_operators)
        Next t
    End If
    If perms_backup_operators <> "***" Then
        For t = 0 To ManageUserGroups.ReturnNumberOfSubGroups(ManageUserGroups.ReturnNTGroupsArrayIndex("Backup Operators")) - 1
            perms = perms + " " + Chr(34) + ManageUserGroups.ReturnSubGroupName(ManageUserGroups.ReturnNTGroupsArrayIndex("Backup Operators"), t) + Chr(34) + ":" + GetNTFSPermissions(perms_backup_operators)
        Next t
    End If
    If perms_print_operators <> "***" Then
        For t = 0 To ManageUserGroups.ReturnNumberOfSubGroups(ManageUserGroups.ReturnNTGroupsArrayIndex("Print Operators")) - 1
            perms = perms + " " + Chr(34) + ManageUserGroups.ReturnSubGroupName(ManageUserGroups.ReturnNTGroupsArrayIndex("Print Operators"), t) + Chr(34) + ":" + GetNTFSPermissions(perms_print_operators)
        Next t
    End If
    If perms_power_users <> "***" Then
        For t = 0 To ManageUserGroups.ReturnNumberOfSubGroups(ManageUserGroups.ReturnNTGroupsArrayIndex("Power Users")) - 1
            perms = perms + " " + Chr(34) + ManageUserGroups.ReturnSubGroupName(ManageUserGroups.ReturnNTGroupsArrayIndex("Power Users"), t) + Chr(34) + ":" + GetNTFSPermissions(perms_power_users)
        Next t
    End If

    If perms_SYSTEM <> "***" Then
        perms = perms + " SYSTEM:" + GetNTFSPermissions(perms_SYSTEM)
    End If
    If perms_EVERYONE <> "***" Then
        perms = perms + " EVERYONE:" + GetNTFSPermissions(perms_EVERYONE)
    End If
    If perms_AUTHENTICATED_USERS <> "***" Then
        perms = perms + " " + Chr(34) + "AUTHENTICATED USERS" + Chr(34) + ":" + GetNTFSPermissions(perms_AUTHENTICATED_USERS)
    End If
    If perms_CREATOR_OWNER <> "***" Then
        perms = perms + " " + Chr(34) + "CREATOR OWNER" + Chr(34) + ":" + GetNTFSPermissions(perms_CREATOR_OWNER)
    End If
    If perms_NETWORK <> "***" Then
        perms = perms + " NETWORK:" + GetNTFSPermissions(perms_NETWORK)
    End If
    If perms_INTERACTIVE <> "***" Then
        perms = perms + " INTERACTIVE:" + GetNTFSPermissions(perms_INTERACTIVE)
    End If


    Select Case operation
        Case "SET FOLDER PERMISSIONS"
            ts.writeline ("echo 'Setting NTFS permissions on " & filepath & "' >> " & log_file_name)
            ts.writeline ("XCACLS " & Chr(34) & filepath & Chr(34) & perms & options & " /C /Y >> " & log_file_name)

        Case "SET FILE PERMISSIONS"
            filepath = filepath + filename
            ts.writeline ("echo 'Setting NTFS permissions on " & filepath & "' >> " & log_file_name)
            ts.writeline ("XCACLS " & Chr(34) & filepath & Chr(34) & perms & options & " /C /Y >> " & log_file_name)

        Case "DELETE FILE"
            filepath = filepath + filename
            ts.writeline ("echo 'Deleting " & filepath & "' >> " & log_file_name)
            ts.writeline ("DELETE " & Chr(34) & filepath & Chr(34) & " >> " & log_file_name)

        Case "DELETE FOLDER"
            ts.writeline ("echo 'Deleting " & foldername & "' >> " & log_file_name)
            ts.writeline ("RMDIR /S /Q " & Chr(34) & foldername & Chr(34) & " >> " & log_file_name)

        Case "MOVE FILE"
            filepath = filepath + filename
            ts.writeline ("echo 'Moving " & Chr(34) & filepath & Chr(34) & " to " & Chr(34) & destination_folder & Chr(34) & "' >> " & log_file_name)
            ts.writeline ("XCOPY " & Chr(34) & filepath & Chr(34) & " " & Chr(34) & destination_folder & Chr(34) & " >> " & log_file_name)
            ts.writeline ("DEL " & Chr(34) & filepath & Chr(34) & " >> " & log_file_name)
    End Select
    
    ts.writeline
    
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Create_Autologon_File()
'
' This procedure creates a .ini file. This .ini file sets the next batch
' file to run after an automatic logon.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Create_Autologon_File(inifilename As String, _
                          nextbatchfile As String, _
                          administratoraccount, _
                          administratorpassword)
    Dim fso2, ts2

    Set fso2 = CreateObject("Scripting.FileSystemObject")
    Set ts2 = fso2.opentextfile(inifilename, 2, True)

    Call Write_To_Regini_file(ts2, "ADD KEY", "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon\", "DefaultUserName", "REG_SZ", administratoraccount, 0, 0, 0, 0, 0, 0, 0, 0)
    Call Write_To_Regini_file(ts2, "ADD KEY", "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon\", "DefaultPassword", "REG_SZ", administratorpassword, 0, 0, 0, 0, 0, 0, 0, 0)
    Call Write_To_Regini_file(ts2, "ADD KEY", "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon\", "AutoAdminLogon", "REG_SZ", "1", 0, 0, 0, 0, 0, 0, 0, 0)

    ' File to run after the reboot
    Call Write_To_Regini_file(ts2, "ADD KEY", "HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\RunOnce\", "HardenNT", "REG_SZ", nextbatchfile, 0, 0, 0, 0, 0, 0, 0, 0)

    ts2.Close
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Write_To_Regini()
'
' This function writes an entry to a .ini file (file_handle) in a format
' that is readable by the REGINI.EXE reskit utility
'
' Operation describes what the entry should do:
' -> "ADD KEY"
' -> "DELETE KEY"
' -> "DELETE HIVE"
' -> "SET PERMISSIONS" (up to 8 per hive)
'
' Depending on the operation, not all parameter values are significant
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Write_To_Regini_file(file_handle, _
                         operation As String, _
                         hive As String, _
                         key As String, _
                         datatype As String, _
                         value, _
                         perm1 As Integer, _
                         perm2 As Integer, _
                         perm3 As Integer, _
                         perm4 As Integer, _
                         perm5 As Integer, _
                         perm6 As Integer, _
                         perm7 As Integer, _
                         perm8 As Integer)
                     
    Dim permstring1, permstring2, permstring3, permstring4 As String
    Dim permstring5, permstring6, permstring7, permstring8 As String

    pos = InStr(hive, "\")

    If (pos = 0) Then
        MsgBox "Syntax Error: Registry hive should end with '\'"
        Exit Sub
    End If

    Do
        part_of_hive = Left(hive, pos - 1)
        hive = Right(hive, Len(hive) - pos)
        For i = 0 To Counter - 1
            file_handle.write ("    ")
        Next
        file_handle.write (part_of_hive)
        pos = InStr(hive, "\")
    
        If pos <> 0 Then file_handle.writeline
    
        Counter = Counter + 1
    Loop Until pos = 0

    Select Case operation
        Case "ADD KEY"
            file_handle.WriteBlankLines (1)
            For i = 0 To Counter - 1
                file_handle.write ("    ")
            Next
            file_handle.write (key & " = " & datatype & " " & value)
    
        Case "DELETE KEY"
            file_handle.WriteBlankLines (1)
            For i = 0 To Counter - 1
                file_handle.write ("    ")
            Next
            file_handle.write (key & " = DELETE")
        
        Case "DELETE HIVE"
            file_handle.write (" = DELETE")
        
        Case "SET PERMISSIONS"
            If perm1 = 0 Then permstring1 = "" Else permstring1 = perm1
            If perm2 = 0 Then permstring2 = "" Else permstring2 = " " & perm2
            If perm3 = 0 Then permstring3 = "" Else permstring3 = " " & perm3
            If perm4 = 0 Then permstring4 = "" Else permstring4 = " " & perm4
            If perm5 = 0 Then permstring5 = "" Else permstring5 = " " & perm5
            If perm6 = 0 Then permstring6 = "" Else permstring6 = " " & perm6
            If perm7 = 0 Then permstring7 = "" Else permstring7 = " " & perm7
            If perm8 = 0 Then permstring8 = "" Else permstring8 = " " & perm8

            file_handle.write (" [" & _
                               permstring1 & _
                               permstring2 & _
                               permstring3 & _
                               permstring4 & _
                               permstring5 & _
                               permstring6 & _
                               permstring7 & _
                               permstring8 & "]")

    End Select

    file_handle.WriteBlankLines (2)
End Sub


