Attribute VB_Name = "ListviewOperations"
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Open_Database()
'
' This function opens the Access database used by the HardenNT program and
' returns true if no errors are encountered, false otherwise.
' Parameters:
' - 'conn': adodb connection object - will be used throughout whole program.
' - 'progress': progress bar - used while loading database.
' - 'databasepath': path to database, passed as a string.
' - 'userrightslistview', 'auditinglistview', 'ntfslistview', and
' 'registrylistview': listviews used for the user rights, auditing, ntfs
' and registry entries.
' 'os' and 'platform': comboboxes used for operating system and platform.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function Open_Database(conn As ADODB.connection, _
                       progress As ProgressBar, _
                       databasepath As String, _
                       userrightslistview As ListView, _
                       auditinglistview As ListView, _
                       ntfslistview As ListView, _
                       registrylistview As ListView, _
                       OS As ComboBox, _
                       Platform As ComboBox) As Boolean
    On Error GoTo Load_Database_Error:

    Dim dataconnection As New ADODB.Recordset

    ' Progress Bar
    progress.Max = 17
    progress.value = 0

    ' Test to see if database is not locked, if all tables are present etc...
    conn.Open "Provider=Microsoft.Jet.OLEDB.4.0;" & _
          "Data Source=" & databasepath & ";" & _
          "Persist Security Info=False"

        progress.value = progress.value + 1

    dataconnection.Open "select * from Patches", conn, adOpenDynamic
    dataconnection.Close
    
        progress.value = progress.value + 1
    
    dataconnection.Open "select * from Systems_Affected", conn, adOpenDynamic
    dataconnection.Close
    
        progress.value = progress.value + 1
    
    dataconnection.Open "select * from MS_Fix", conn, adOpenDynamic
    dataconnection.Close
    
        progress.value = progress.value + 1
    
    ' Fill Platform ComboBox
    dataconnection.Open "select Platform_Name from Platform", conn, adOpenDynamic
    FillComboBox dataconnection, Platform, 0
    dataconnection.Close

        progress.value = progress.value + 1

    ' Fill OS ComboBox
    dataconnection.Open "select OS_Name from OS order by OS_ID", conn, adOpenDynamic
    FillComboBox dataconnection, OS, 0
    dataconnection.Close

        progress.value = progress.value + 1

    ' Fill Registry_Listview
    dataconnection.Open "SELECT Registry_Description,      " & _
                                "Registry_Operation_ID,    " & _
                                "Registry_Hive,            " & _
                                "Registry_Key,             " & _
                                "Registry_Data_Type_ID,    " & _
                                "Registry_Value,           " & _
                                "Registry_Permission_1,    " & _
                                "Registry_Permission_2,    " & _
                                "Registry_Permission_3,    " & _
                                "Registry_Permission_4,    " & _
                                "Registry_Permission_5,    " & _
                                "Registry_Permission_6,    " & _
                                "Registry_Permission_7,    " & _
                                "Registry_Permission_8,    " & _
                                "Registry_Helpfile,        " & _
                                "Registry_ID               " & _
                        "FROM Registry ORDER BY Registry_Description", conn, adOpenDynamic
    FillListView dataconnection, registrylistview
    dataconnection.Close
    
        progress.value = progress.value + 1
    
    dataconnection.Open "SELECT * FROM Registry_Permission", conn, adOpenDynamic
    FillFromOther "Registry_Permission_1", registrylistview, dataconnection, "Registry_Permission_Name"
    FillFromOther "Registry_Permission_2", registrylistview, dataconnection, "Registry_Permission_Name"
    FillFromOther "Registry_Permission_3", registrylistview, dataconnection, "Registry_Permission_Name"
    FillFromOther "Registry_Permission_4", registrylistview, dataconnection, "Registry_Permission_Name"
    FillFromOther "Registry_Permission_5", registrylistview, dataconnection, "Registry_Permission_Name"
    FillFromOther "Registry_Permission_6", registrylistview, dataconnection, "Registry_Permission_Name"
    FillFromOther "Registry_Permission_7", registrylistview, dataconnection, "Registry_Permission_Name"
    FillFromOther "Registry_Permission_8", registrylistview, dataconnection, "Registry_Permission_Name"
    dataconnection.Close
    
        progress.value = progress.value + 1
    
    dataconnection.Open "SELECT * FROM Registry_Operation", conn, adOpenDynamic
    Call FillFromOther("Registry_Operation_ID", registrylistview, dataconnection, "Registry_Operation_Name")
    dataconnection.Close
    
        progress.value = progress.value + 1
    
    dataconnection.Open "SELECT * FROM Registry_Data_Type", conn, adOpenDynamic
    Call FillFromOther("Registry_Data_Type_ID", registrylistview, dataconnection, "Registry_Data_Type_Name")
    dataconnection.Close

        progress.value = progress.value + 1

    ' Fill User_Rights_Listview
    
    dataconnection.Open "SELECT User_Rights.User_Rights_Description, " & _
                        "       User_Rights_Operation.User_Rights_Operation_Name, " & _
                        "       NT_User_Groups.NT_User_Group_Name, " & _
                        "       User_Rights_Privileges.User_Rights_Privilege_Name, " & _
                        "       User_Rights_Privileges.User_Rights_Privilege_Description, " & _
                        "       User_Rights_Privileges.User_Rights_Advanced, " & _
                        "       User_Rights.User_Rights_ID " & _
                        "FROM User_Rights_Privileges INNER JOIN " & _
                        " (" & _
                        " User_Rights_Operation INNER JOIN " & _
                        "  (" & _
                        "  NT_User_Groups INNER JOIN " & _
                        "   User_Rights ON NT_User_Groups.NT_User_Group_ID = User_Rights.User_Rights_Group_ID) ON User_Rights_Operation.User_Rights_Operation_ID = User_Rights.User_Rights_Operation_ID) ON User_Rights_Privileges.User_Rights_Privilege_ID = User_Rights.User_Rights_Privilege_ID", _
                        conn, adOpenDynamic
    FillListView dataconnection, userrightslistview
    dataconnection.Close
    
        progress.value = progress.value + 1

    ' Fill Auditing_Listview
    dataconnection.Open "SELECT Auditing_Description, Auditing_Name, Audit_Level_ID, Auditing_ID FROM Auditing", conn, adOpenDynamic
    FillListView dataconnection, auditinglistview
    dataconnection.Close
    
        progress.value = progress.value + 1
    
    dataconnection.Open "SELECT * FROM Audit_Level", conn, adOpenDynamic
    FillFromOther "Audit_Level_ID", auditinglistview, dataconnection, "Audit_Level_Description"
    dataconnection.Close

        progress.value = progress.value + 1

    ' Fill NTFS_Listview
    dataconnection.Open "SELECT  NTFS_Description,                    " & _
                                "NTFS_Operation_ID,                   " & _
                                "NTFS_Folder,                         " & _
                                "NTFS_File,                           " & _
                                "NTFS_Recursively,                    " & _
                                "NTFS_Set_Perms_Type_ID,              " & _
                                "NTFS_Permission_Administrators,      " & _
                                "NTFS_PERMISSION_SYSTEM,              " & _
                                "NTFS_Permission_Users,               " & _
                                "NTFS_Permission_Guests,              " & _
                                "NTFS_PERMISSION_EVERYONE,            " & _
                                "NTFS_PERMISSION_AUTHENTICATED_USERS, " & _
                                "NTFS_PERMISSION_CREATOR_OWNER,       " & _
                                "NTFS_PERMISSION_NETWORK,             " & _
                                "NTFS_PERMISSION_INTERACTIVE,         " & _
                                "NTFS_Permission_Server_Operators,    " & _
                                "NTFS_Permission_Account_Operators,   " & _
                                "NTFS_Permission_Backup_Operators,    " & _
                                "NTFS_Permission_Print_Operators,     " & _
                                "NTFS_Permission_Power_Users,         " & _
                                "NTFS_Helpfile,                       " & _
                                "NTFS_ID                              " & _
                        "FROM NTFS ORDER BY NTFS_ID", conn, adOpenDynamic
    FillListView dataconnection, ntfslistview
    dataconnection.Close

        progress.value = progress.value + 1

    dataconnection.Open "SELECT * FROM NTFS_Permission", conn, adOpenDynamic
    Call FillFromOther("NTFS_Permission_Administrators", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_PERMISSION_SYSTEM", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_Permission_Users", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_Permission_Guests", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_PERMISSION_EVERYONE", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_PERMISSION_AUTHENTICATED_USERS", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_PERMISSION_CREATOR_OWNER", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_PERMISSION_NETWORK", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_PERMISSION_INTERACTIVE", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_Permission_Server_Operators", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_Permission_Account_Operators", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_Permission_Backup_Operators", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_Permission_Print_Operators", ntfslistview, dataconnection, "NTFS_Permission_Name")
    Call FillFromOther("NTFS_Permission_Power_Users", ntfslistview, dataconnection, "NTFS_Permission_Name")
    dataconnection.Close

        progress.value = progress.value + 1

    dataconnection.Open "SELECT * FROM NTFS_Operation", conn, adOpenDynamic
    Call FillFromOther("NTFS_Operation_ID", ntfslistview, dataconnection, "NTFS_Operation_Name")
    dataconnection.Close
    
        progress.value = progress.value + 1
    
    dataconnection.Open "SELECT * FROM NTFS_Set_Perms_Type", conn, adOpenDynamic
    Call FillFromOther("NTFS_Set_Perms_Type_ID", ntfslistview, dataconnection, "NTFS_Set_Perms_Type_Name")
    dataconnection.Close

        progress.value = progress.value + 1

    ' Database is successfully loaded !

    ' Initialize ManageUserGroups and SelectProfile form
    ManageUserGroups.Initialize (databasepath)
    SelectProfile.Initialize (databasepath)

    Open_Database = True
    Exit Function
    
Load_Database_Error:
    Call Error_Message("Load_Database()")
    progress.value = 0
    Open_Database = False
End Function

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SelectPatchesAndApplyProfiles()
'
' This function is called from the main program (if the database has been
' loaded) if a user selects an operating system, platform or service pack
' level.
' The function fills the patches listview with the patches the user will
' be able to select (depending on OS, platform and SP level) and then applies
' the selected profile(s) to the listview.
' Parameters:
' - 'lsvw' : listview that will contain the patches.
' - 'conn' : connection to database (must be opened by calling program).
' - 'platformtext', 'ostext' and 'spleveltext' : contains the name of the
' platform selected, the os selected and the SP level selected.
' - 'profileID' : ID of the profile that will be applied to the listview
' after it has been filled.
' - 'profile_loaded' : boolean, TRUE if one or more profiles have been
' selected.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub SelectPatchesAndApplyProfiles(lsvw As ListView, _
                                  conn As ADODB.connection, _
                                  platformtext As String, _
                                  ostext As String, _
                                  spleveltext As String, _
                                  profileid As Integer, _
                                  profile_loaded As Boolean, _
                                  interaction_required As Boolean)
'    On Error GoTo SelectPatchesAndApplyProfiles_Error:

    Dim tempdataconnection As New ADODB.Recordset

    If platformtext <> "" And ostext <> "" And spleveltext <> "" Then
        Call FillPatchesListView(lsvw, _
                                 conn, _
                                 ostext, _
                                 platformtext, _
                                 spleveltext, _
                                 interaction_required)
        If profile_loaded Then
            tempdataconnection.Open "SELECT MS_Fix_ID FROM Link_Profile_MS_Fix " & _
                                    "WHERE Profile_ID = " & profileid, conn, adOpenDynamic
            Call Check_ListItems_MS_Fix(lsvw, tempdataconnection)
            tempdataconnection.Close
        End If
    End If

    Exit Sub

SelectPatchesAndApplyProfiles_Error:
    Call Error_Message("SelectPatchesAndApplyProfiles()")
End Sub


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' FillPatchesListView()
'
' This function fills the patches listview with a list of patches, depending
' on the operating system, platform and service pack level selected.
' When the listview is filled, SubstituteSystemsAffected() is called to make
' sure the list of systems affected keys is changed to text.
' Parameters:
' - 'lsvw' : the listview that will be filled with patches.
' - 'conn' : connection to database.
' - 'ostext', 'platformtext' and 'splevel' : the operating system, platform
' and service pack level selected by the user.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub FillPatchesListView(lsvw As ListView, _
                        conn As ADODB.connection, _
                        ostext As String, _
                        platformtext As String, _
                        spleveltext As String, _
                        interaction_required As Boolean)
'    On Error GoTo FillPatchesListview_Error:

    Dim selectstring As String
    Dim tempdataconnection As New ADODB.Recordset

    selectstring = "SELECT   MS_Fix.MS_Fix_ID, " & _
                            "MS_Fix.MS_Fix_Description, " & _
                            "MS_Fix.MS_Fix_Release_Date, " & _
                            "Patches.Patch_Regression_Tested, " & _
                            "Patches.Patch_Executable, " & _
                            "Patches.Patch_Note, " & _
                            "MS_Fix.MS_Fix_Systems_Affected " & _
                   "FROM Platform INNER JOIN " & _
                   " (OS INNER JOIN " & _
                   "  (MS_Fix INNER JOIN " & _
                   "   Patches " & _
                   "   ON MS_Fix.MS_Fix_ID = Patches.MS_Fix_ID" & _
                   "  ) " & _
                   "  ON OS.OS_ID = Patches.OS_Name " & _
                   " ) " & _
                   " ON Platform.Platform_ID = Patches.Platform_Name " & _
                   "WHERE ((((OS.OS_Name)              = '" & ostext & "') AND " & _
                   "        ((Platform.Platform_Name)  = '" & platformtext & "') AND " & _
                   "        (((Patches.Patch_SP_Fixed) = '-1') OR" & _
                   "         ((Patches.Patch_SP_Fixed) > '" & spleveltext & "'))) "

    If interaction_required = True Then
        selectstring = selectstring & _
                       " ) ORDER BY MS_Fix.MS_Fix_ID;"

    Else
        selectstring = selectstring & _
                       " AND (Patches.Patch_Interaction) = " & interaction_required & ") " & _
                       " ORDER BY MS_Fix.MS_Fix_ID;"
    End If

    tempdataconnection.Open selectstring, conn, adOpenDynamic
    FillListView tempdataconnection, lsvw
    tempdataconnection.Close
    
    tempdataconnection.Open "SELECT * FROM Systems_Affected", conn, adOpenDynamic
    Call SubstituteSystemsAffected(lsvw, tempdataconnection, 7)
    Call lvAutosizeControl(lsvw)
    tempdataconnection.Close

    Exit Sub

FillPatchesListview_Error:
    Call Error_Message("FillPatchesListview()")
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Check_ListItems()
'
' This function is called when applying one or more profiles: datacon
' contains IDs of listview items that should be checked in the selected
' profile(s). It's important to know that the last column of the 'lsvw'
' listview should contain the IDs of the listview entries as to allow
' Check_ListTtems() to compare the column IDs with the IDs that resulted
' from a database query (datacon).
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Check_ListItems(lsvw As ListView, datacon As ADODB.Recordset)
    On Error GoTo Check_ListItems:

    Dim tempcolumn As Integer

    tempcolumn = lsvw.ColumnHeaders.Count - 1
    If Not datacon.EOF Then datacon.MoveFirst
    Do While Not datacon.EOF
        For i = 1 To lsvw.ListItems.Count
            If lsvw.ListItems(i).SubItems(tempcolumn) = datacon.Fields.Item(0).value Then
                lsvw.ListItems(i).Checked = True
            End If
        Next
        datacon.MoveNext
    Loop

    Exit Sub

Check_ListItems:
    Call Error_Message("Check_ListItems()")
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Check_ListItems_MS_Fix()
'
' This function is called when applying one or more profiles to the
' patches listview. Note that it's impossible to apply profiles to the
' patches listview via Check_ListItems(), since the patches listview
' stores the IDs in the first column of the listview, not the last.
' Also, the patches profile(s) are not applied when a profile is loaded,
' but rather whenever a user changes the operating system, platform or
' service pack level.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Check_ListItems_MS_Fix(lsvw As ListView, datacon As ADODB.Recordset)
    If Not datacon.EOF Then datacon.MoveFirst
    Do While Not datacon.EOF
        For i = 1 To lsvw.ListItems.Count
            If lsvw.ListItems(i).text = datacon.Fields.Item(0).value Then
                lsvw.ListItems(i).Checked = True
            End If
        Next
        datacon.MoveNext
    Loop
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SubstituteSystemsAffected()
'
' This subroutine replaces the numbers (=IDs) of the systems affected
' column (patches listview) by their user-friendly names.
' ex. "1,3" -> "NT 4.0 Server, Internet Information Server 4.0"
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub SubstituteSystemsAffected(P_Listview As ListView, _
                              systems_affected_dataconnection As ADODB.Recordset, _
                              columnindex As Integer)
    Dim t As Integer

    For t = 1 To P_Listview.ListItems.Count
        'MsgBox patches_listview.ListItems(t).SubItems(columnindex - 1)
        P_Listview.ListItems(t).SubItems(columnindex - 1) = AnalyzeAffectedSystemsLine(P_Listview.ListItems(t).SubItems(columnindex - 1), systems_affected_dataconnection)
    Next

End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' AnalyzeAffectedSystemsLine()
'
' Called by SubstituteSystemsAffected() to replace the systems affected
' numbers by user-friendly names.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function AnalyzeAffectedSystemsLine(systemsline As String, _
                                    sRec As ADODB.Recordset) As String
    Dim t, position As Integer
    Dim startposition As Integer
    Dim returnline As String
    Dim commacounter As Integer
    Dim splitarray() As String
    
    If systemsline = "" Then
        AnalyzeAffectedSystemsLine = ""
        Exit Function
    End If
    
    ' Count number of commas in systemline
    commacounter = 0
    startposition = 1
    position = InStr(startposition, systemsline, ",")
    Do While position <> 0
        commacounter = commacounter + 1
        startposition = position + 1
        position = InStr(startposition, systemsline, ",")
    Loop
    
    ' Add 1 to get the number of "Systems Affected" numbers to convert to strings, then redim array
    commacounter = commacounter
    ReDim splitarray(commacounter)
    
    splitarray = Split(systemsline, ",")
    
    ' Convert to strings
    returnline = SystemsNumericToString(Val(splitarray(0)), sRec)
    For t = 1 To commacounter
        returnline = returnline + ", " + SystemsNumericToString(Val(splitarray(t)), sRec)
    Next
    
    AnalyzeAffectedSystemsLine = returnline
End Function

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SystemsNumericToString()
'
' Called by AnalyzeAffectedSystemsLine() to get the user-friendly description
' for a numeric value (=Systems Affected ID).
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function SystemsNumericToString(systemnumeric As Integer, sRec As ADODB.Recordset) As String
    If Not sRec.EOF Then sRec.MoveFirst
    Do While Not sRec.EOF
        If sRec.Fields.Item(0) = systemnumeric Then
            SystemsNumericToString = sRec.Fields.Item(1)
            Exit Function
        End If
        sRec.MoveNext
    Loop
    
    ' Error : Not found in database
    SystemsNumericToString = ""
End Function

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Check_Uncheck_All()
'
' Checks or unchecks all items in a listview ('Lview')
' 'check_uncheck': either true or false, depending on whether all items
' should be checked or unchecked.
' 'database_loaded': used to check if the security database has already
' been loaded.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Check_Uncheck_All(Lview As ListView, _
                      check_uncheck As Boolean, _
                      Database_Loaded As Boolean)
    Dim i As Integer

    If Database_Loaded = True Then
        For i = 1 To Lview.ListItems.Count
            Lview.ListItems(i).Checked = check_uncheck
        Next
    Else
        MsgBox "Load a Database First"
    End If
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' GetListviewColumnIndex()
'
' This function returns the index of the column that has 'text' as the
' ColumnHeaders text of the 'lsvw' listview, or -1 if 'text' was not found.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function GetListviewColumnIndex(lsvw As ListView, text As String) As Integer
    Dim t As Integer

    For t = 1 To lsvw.ColumnHeaders.Count
        If lsvw.ColumnHeaders.Item(t).text = text Then
            GetListviewColumnIndex = t
            Exit Function
        End If
    Next

    GetListviewColumnIndex = -1
End Function

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' StringInListviewExists()
'
' Returns TRUE if the 'searchfor' string exists in the 'lsvw' listview,
' FALSE otherwise.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function StringInListviewExists(searchfor As String, lsvw As ListView) As Boolean
    Dim t As Integer
    
    For t = 1 To lsvw.ListItems.Count
        If lsvw.ListItems(t).text = searchfor Then
            StringInListviewExists = True
            Exit Function
        End If
    Next
    
    StringInListviewExists = False
End Function

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Load_Profile()
'
' Called when a profile needs to be applied to the different listview.
' First all the checkboxes are cleared, then the profile is applied to
' every listview by calling Apply_Profile().
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Load_Profile(plistview As ListView, _
                 urlistview As ListView, _
                 alistview As ListView, _
                 nlistview As ListView, _
                 rlistview As ListView, _
                 connection As ADODB.connection, _
                 dataconnection As ADODB.Recordset, _
                 profileid As Integer, _
                 numberofprofiles As Integer, _
                 Profiles As SelectProfile)
    Dim i As Integer

    ' Clear 'selected' checkboxes of previously selected profiles
    Call Clear_Selected_Checkboxes(plistview)
    Call Clear_Selected_Checkboxes(urlistview)
    Call Clear_Selected_Checkboxes(alistview)
    Call Clear_Selected_Checkboxes(nlistview)
    Call Clear_Selected_Checkboxes(rlistview)

    ' Now Apply All Profiles
    For i = 1 To numberofprofiles
        dataconnection.Open "SELECT Profile_ID FROM Profile WHERE Profile_Name = '" & Profiles.GetArray(i) & "'", connection, adOpenDynamic
        dataconnection.MoveFirst
        
        If Not dataconnection.EOF Then
            profileid = dataconnection.Fields(0).value
        Else
            MsgBox "Error. Possible corruption in database entries ('Profile' table)"
            Exit Sub
        End If
        
        dataconnection.Close

        Call Apply_Profile(plistview, _
                           urlistview, _
                           alistview, _
                           nlistview, _
                           rlistview, _
                           connection, _
                           dataconnection, _
                           profileid)
    Next
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Write_Profiles()
'
' This subroutine is called from the main program/form to save the selected
' profile(s). First, the database is searched to determine if the profile
' name already exists. If so, the existing profile is deleted from the
' database and then added again.
' Parameters:
'
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub Write_Profiles(conn As ADODB.connection, _
                   dataconnection As ADODB.Recordset, _
                   profiletext As String, _
                   profilescancelclicked As Boolean, _
                   profiledescription As String, _
                   plistview As ListView, _
                   urlistview As ListView, _
                   alistview As ListView, _
                   ntfslistview As ListView, _
                   rlistview As ListView)
    On Error GoTo Write_Profiles_Error:

    Dim profileid As Integer
    Dim Profiles_Description As String

    ' First, get Profile_ID
    dataconnection.Open "SELECT Profile_ID FROM Profile " & _
                        "WHERE Profile_Name = '" & profiletext & "'", conn, adOpenDynamic
    If Not dataconnection.EOF Then
        ' Get profile ID
        dataconnection.MoveFirst
        profileid = dataconnection.Fields.Item(0).value
        dataconnection.Close

        ' Next, delete all occurences of this profile
        ' -> Profile table
        dataconnection.Open "DELETE FROM Profile WHERE " & _
                            "Profile_Name = '" & profiletext & "'", conn, adOpenDynamic
        ' -> Link_... Tables
        dataconnection.Open "DELETE FROM Link_Profile_User_Rights " & _
                            "WHERE Profile_ID = " & profileid, conn, adOpenDynamic
        dataconnection.Open "DELETE FROM Link_Profile_Auditing " & _
                            "WHERE Profile_ID = " & profileid, conn, adOpenDynamic
        dataconnection.Open "DELETE FROM Link_Profile_MS_Fix " & _
                            "WHERE Profile_ID = " & profileid, conn, adOpenDynamic
        dataconnection.Open "DELETE FROM Link_Profile_NTFS " & _
                            "WHERE Profile_ID = " & profileid, conn, adOpenDynamic
        dataconnection.Open "DELETE FROM Link_Profile_Registry " & _
                            "WHERE Profile_ID = " & profileid, conn, adOpenDynamic
    Else
        ' This profile doesn't exist
        dataconnection.Close
    End If
        
    ' Lastly, Write everything to Profile table
    ' -> Profile table
    If profilescancelclicked Then
        Profiles_Description = ""
    Else
        Profiles_Description = profiledescription
    End If
    dataconnection.Open "INSERT INTO Profile (Profile_Name, Profile_Description) VALUES ('" & profiletext & "', '" & Profiles_Description & "')", conn, adOpenDynamic

    ' Get Profile_ID
    dataconnection.Open "SELECT Profile_ID FROM Profile " & _
                        "WHERE Profile_Name = '" & profiletext & "'", conn, adOpenDynamic
    dataconnection.MoveFirst
    profileid = dataconnection.Fields.Item(0).value
    dataconnection.Close
        
    ' -> Link_... tables
    Call Write_Profile_MS_Fix("MS_Fix", plistview, conn, profileid)
    Call Write_Profile("User_Rights", urlistview, conn, profileid)
    Call Write_Profile("Auditing", alistview, conn, profileid)
    Call Write_Profile("NTFS", ntfslistview, conn, profileid)
    Call Write_Profile("Registry", rlistview, conn, profileid)

    Exit Sub

Write_Profiles_Error:
    Call Error_Message("Write_Profiles()")
End Sub

